#include <asf.h>
#include <stdio.h>
#include <string.h>
#include <conf_example.h>
#include <util/delay.h>

static volatile int16_t last_temperature;
uint8_t tx_buf3[]="/";
uint8_t tx_buf4[] = "0123456789";
uint8_t tx_buf5[]="SRU";
char out_str[OUTPUT_STR_SIZE];
int a,b;

/**
 * \brief Callback function for ADC interrupts
 *
 * \param adc Pointer to ADC module.
 * \param ch_mask ADC channel mask.
 * \param result Conversion result from ADC channel.
 */
static void adc_handler(ADC_t *adc, uint8_t ch_mask, adc_result_t result)
{
	#ifdef CONF_BOARD_OLED_UG_2832HSWEG04
		gfx_mono_draw_filled_rect(0,0,128,32,GFX_PIXEL_CLR);
	#endif
	int32_t temperature;
	

	/* Compute current temperature in Celsius, based on linearization
	 * of the temperature sensor adc data.
	 */
	
	if (result > 697) {
		temperature = (int8_t)((-0.0295 * result) + 40.5);
	} if (result > 420) {
		temperature = (int8_t)((-0.0474 * result) + 53.3);
	} else {
		temperature = (int8_t)((-0.0777 * result) + 65.1);
	}
	
	last_temperature = temperature;
	
	_delay_ms(500);
	
	/*Break */
	
	usart_putchar(USART_SERIAL_J4,tx_buf3[0]);
	usart_putchar(USART_SERIAL_J1,tx_buf3[0]);
	
	/*Mode U: USB ; R:Radio ; S: Serial*/
	
	usart_putchar(USART_SERIAL_J4,tx_buf5[2]);
	usart_putchar(USART_SERIAL_J1,tx_buf5[0]);
	
	/*ID*/
	
	usart_putchar(USART_SERIAL_J4,tx_buf4[0]);
	usart_putchar(USART_SERIAL_J1,tx_buf4[0]);
	
	usart_putchar(USART_SERIAL_J4,tx_buf4[0]);
	usart_putchar(USART_SERIAL_J1,tx_buf4[0]);
	
	usart_putchar(USART_SERIAL_J4, tx_buf4[2]);//5
	usart_putchar(USART_SERIAL_J1,tx_buf4[2]);
	
	/*carte fille*/
	
	usart_putchar(USART_SERIAL_J4,tx_buf4[1]);
	usart_putchar(USART_SERIAL_J1,tx_buf4[1]);
	
	/*Carte petite fille*/
	
	usart_putchar(USART_SERIAL_J4,tx_buf4[1]);
	usart_putchar(USART_SERIAL_J1,tx_buf4[1]);
	
	/*Data*/
	
	a=last_temperature/10;
	b=last_temperature%10;
	
	usart_putchar(USART_SERIAL_J4,tx_buf4[0]);
	usart_putchar(USART_SERIAL_J1,tx_buf4[0]);
	
	usart_putchar(USART_SERIAL_J4,tx_buf4[a]);
	usart_putchar(USART_SERIAL_J1,tx_buf4[a]);
	
	usart_putchar(USART_SERIAL_J4,tx_buf4[b]);
	usart_putchar(USART_SERIAL_J1,tx_buf4[b]);
	
	// Write temperature to display
	
	snprintf(out_str, OUTPUT_STR_SIZE, "Temperature: %4d C", last_temperature);
	gfx_mono_draw_string(out_str, 0, 0, &sysfont);
	snprintf(out_str, OUTPUT_STR_SIZE, "Don't touch this !!");
	gfx_mono_draw_string(out_str, 0, 10, &sysfont);
	snprintf(out_str, OUTPUT_STR_SIZE, "Carte B Pdi c21 d20");
	gfx_mono_draw_string(out_str, 0, 20, &sysfont);
	
	// Start next conversion.
	
	adc_start_conversion(adc, ch_mask);
}

int main(void)
{
	board_init();
	sysclk_init();
	
	static usart_rs232_options_t USART_SERIAL_OPTIONS_J1 = {
		.baudrate = USART_SERIAL_J1_BAUDRATE,
		.charlength = USART_SERIAL_CHAR_LENGTH,
		.paritytype = USART_SERIAL_PARITY,
		.stopbits = USART_SERIAL_STOP_BIT
	};
	
	static usart_rs232_options_t USART_SERIAL_OPTIONS_J4 = {
		.baudrate = USART_SERIAL_J4_BAUDRATE,
		.charlength = USART_SERIAL_CHAR_LENGTH,
		.paritytype = USART_SERIAL_PARITY,
		.stopbits = USART_SERIAL_STOP_BIT
	};
	
	usart_init_rs232(USART_SERIAL_J1, &USART_SERIAL_OPTIONS_J1);
	usart_init_rs232(USART_SERIAL_J4, &USART_SERIAL_OPTIONS_J4);
	
	struct adc_config         adc_conf;
	struct adc_channel_config adcch_conf;
	
	sleepmgr_init();
	irq_initialize_vectors();
	cpu_irq_enable();
	gfx_mono_init();
	
	// Enable backlight if display type is not OLED
	
	#ifndef CONF_BOARD_OLED_UG_2832HSWEG04
		ioport_set_pin_high(LCD_BACKLIGHT_ENABLE_PIN);
	#endif
	
	// Initialize configuration structures
	
	adc_read_configuration(&ADCA, &adc_conf);
	adcch_read_configuration(&ADCA, ADC_CH0, &adcch_conf);

	/* Configure the ADC module:
	 * - unsigned, 12-bit results
	 * - VCC voltage reference
	 * - 200 kHz maximum clock rate
	 * - manual conversion triggering
	 * - temperature sensor enabled
	 * - callback function
	 */
    
	adc_set_conversion_parameters(&adc_conf, ADC_SIGN_ON, ADC_RES_12,ADC_REF_VCC);
	adc_set_clock_rate(&adc_conf, 200000UL);
	adc_set_conversion_trigger(&adc_conf, ADC_TRIG_MANUAL, 1, 0);
	adc_enable_internal_input(&adc_conf, ADC_INT_TEMPSENSE);

	adc_write_configuration(&ADCA, &adc_conf);
	adc_set_callback(&ADCA, &adc_handler);

	/* Configure ADC channel 0:
	 * - single-ended measurement from temperature sensor
	 * - interrupt flag set on completed conversion
	 * - interrupts disabled
	 */
	
	adcch_set_input(&adcch_conf, ADCCH_POS_PIN1, ADCCH_NEG_NONE,1);
	adcch_set_interrupt_mode(&adcch_conf, ADCCH_MODE_COMPLETE);
	adcch_enable_interrupt(&adcch_conf);
	adcch_write_configuration(&ADCA, ADC_CH0, &adcch_conf);

	// Enable the ADC and start the first conversion
	
	adc_enable(&ADCA);
	adc_start_conversion(&ADCA, ADC_CH0);
	while(1) {
		
		// Sleep until ADC interrupt triggers
		sleepmgr_enter_sleep();
	}
}
